/*-
 * Copyright (C)2004 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2004 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: tstsock.c,v 1.1 2004/03/01 07:38:08 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sysexits.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <string.h>
#include <netdb.h>
#include <stdio.h>
#define STCP 0
#define SUDP 1
#define SUNX 2
#define BUFLEN 64

void
usage(int ex) {
    exit(ex);
}

int
main(int argc, char ** argv) {
    int c, res;
    int socktype;
    char *bindaddr;
    struct addrinfo hints, *r, *sa;
    char *host, *port;
    struct timeval timeout;
    u_char buf[BUFLEN];

    bzero(buf, BUFLEN);
    bindaddr=NULL;
    socktype = STCP;
    bzero(&timeout, sizeof(timeout));
    while ((c = getopt(argc, argv, "b:TUX")) != EOF)
        switch (c) {
        case 'b': bindaddr = optarg; break;
        case 't': timeout.tv_sec = strtol(optarg, NULL, 0);
        case 'T': socktype = STCP;   break;
        case 'U': socktype = SUDP;   break;
        case 'X': socktype = SUNX;   break;
        default: usage(EX_USAGE);
        }
    argc -= optind;
    argv += optind;
    host = argv[0];
    port = NULL;
    switch (argc) {
    case 1:
        port = rindex(argv[0], ':');
        if  (!port) usage(EX_USAGE);
        *port++ = '\0';
        break;
    case 2:
        port = argv[1];
        break;
    default: usage(EX_USAGE);
    }
    bzero(&hints, sizeof(hints));
    switch (socktype) {
    case STCP:
        hints.ai_family = PF_INET;
        hints.ai_socktype = SOCK_STREAM;
        hints.ai_protocol = IPPROTO_TCP;
        break;
    case SUDP:
        hints.ai_family = PF_INET;
        hints.ai_socktype = SOCK_DGRAM;
        hints.ai_protocol = IPPROTO_UDP;
        break;
    case SUNX:
    default: usage(EX_USAGE);
    }
    if  ((res = getaddrinfo(host, port, &hints, &r))) exit(res);
    if  ((c = socket(r->ai_family, r->ai_socktype, r->ai_protocol)) < 0) exit(c);
    if  (timeout.tv_sec) {
        if  ((res = setsockopt(c, SOL_SOCKET, SO_SNDTIMEO, &timeout, sizeof(timeout)))) exit(res);
        if  ((res = setsockopt(c, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(timeout)))) exit(res);
    }
    res = 1;
    sa = r;
    while (res && sa) {
        switch (socktype) {
        case STCP:
            res = connect(c, sa->ai_addr, sa->ai_addrlen);
            break;
        case SUDP:
            res = sendto(c, buf, BUFLEN, 0, sa->ai_addr, sa->ai_addrlen);
            if  (res > 0) res = 0;
            break;
        case SUNX:
        default: usage(EX_USAGE);
        }
        sa = sa->ai_next;
        close(c);
    }
    exit(res);
}
